@@ -61,6 +61,7 @@
 #include "cfgParser.h"
 #include "pgmParser.h"
 #include "program.h"
+#include "midnam_lv2.h"
 
 typedef enum {
   B3S_MIDIIN = 0,
@@ -73,6 +74,7 @@ typedef struct {
   LV2_Atom_Forge forge;
   LV2_Atom_Forge_Frame frame;
   LV2_Worker_Schedule* schedule;
+  LV2_Midnam*          midnam;
 
   const LV2_Atom_Sequence* midiin;
   LV2_Atom_Sequence* midiout;
@@ -286,6 +288,9 @@ static void mctl_cb(int fnid, const char *fn, unsigned char val, midiCCmap *mm,
   if (b3s->midiout && fn && !b3s->suspend_ui_msg) {
     forge_kvcontrolmessage(&b3s->forge, &b3s->uris, fn, (int32_t) val);
   }
+  if (b3s->midnam && fn && !strcmp (fn, "special.midimap")) {
+    b3s->midnam->update (b3s->midnam->handle);
+  }
 }
 
 static void rc_cb(int fnid, const char *key, const char *kv, unsigned char val, void *arg) {
@@ -816,6 +821,9 @@ postrun (B3S* b3s)
     b3s->schedule->schedule_work(b3s->schedule->handle, sizeof(struct worknfo), &w);
     b3s->update_gui_now = 1;
     b3s->swap_instances = 0;
+    if (b3s->midnam) {
+      b3s->midnam->update (b3s->midnam->handle);
+    }
   }
 }
 
@@ -874,6 +882,8 @@ instantiate(const LV2_Descriptor*     descriptor,
       b3s->map = (LV2_URID_Map*)features[i]->data;
     } else if (!strcmp(features[i]->URI, LV2_WORKER__schedule)) {
       b3s->schedule = (LV2_Worker_Schedule*)features[i]->data;
+    } else if (!strcmp (features[i]->URI, LV2_MIDNAM__update)) {
+      b3s->midnam = (LV2_Midnam*)features[i]->data;
     }
   }
 
@@ -1053,6 +1063,7 @@ run(LV2_Handle instance, uint32_t n_samples)
 {
   B3S* b3s = (B3S*)instance;
   float* audio[2];
+  bool dirty = false;
 
   audio[0] = b3s->outL;
   audio[1] = b3s->outR;
@@ -1103,12 +1114,14 @@ run(LV2_Handle instance, uint32_t n_samples)
 	  lv2_atom_object_get(obj, b3s->uris.sb3_cckey, &flags, b3s->uris.sb3_ccval, &cmd, 0);
 	  if (cmd && flags) {
 	    midi_uiassign_cc(b3s->inst->midicfg, (const char*)LV2_ATOM_BODY(cmd), ((LV2_Atom_Int*)flags)->body);
+	    dirty = true;
 	  }
 	} else if (obj->body.otype == b3s->uris.sb3_midipgm) {
 	  const LV2_Atom* key = NULL;
 	  lv2_atom_object_get(obj, b3s->uris.sb3_cckey, &key, 0);
 	  if (key) {
 	    installProgram(b3s->inst, ((LV2_Atom_Int*)key)->body);
+	    dirty = true;
 	  }
 	} else if (obj->body.otype == b3s->uris.sb3_midisavepgm) {
 	  const LV2_Atom* pgm = NULL;
@@ -1117,6 +1130,9 @@ run(LV2_Handle instance, uint32_t n_samples)
 	  if (pgm && name) {
 	    saveProgramm(b3s->inst, (int) ((LV2_Atom_Int*)pgm)->body, (char*) LV2_ATOM_BODY(name), 0);
 	    b3s->update_pgm_now = 1;
+	    if (b3s->midnam) {
+	      b3s->midnam->update (b3s->midnam->handle);
+	    }
 	  }
 	} else if (obj->body.otype == b3s->uris.sb3_loadpgm) {
 	  iowork(b3s, obj, CMD_LOADPGM);
@@ -1129,6 +1145,7 @@ run(LV2_Handle instance, uint32_t n_samples)
 	} else if (obj->body.otype == b3s->uris.sb3_cfgstr) {
 	  if (!b3s->inst_offline) {
 	    advanced_config_set(b3s, obj);
+	    dirty = true;
 	  }
 	} else if (obj->body.otype == b3s->uris.sb3_control) {
 	  b3s->suspend_ui_msg = 1;
@@ -1139,6 +1156,7 @@ run(LV2_Handle instance, uint32_t n_samples)
 	    fprintf(stderr, "B3LV2: callMIDIControlFunction(..,\"%s\", %d);\n", k, v);
 #endif
 	    callMIDIControlFunction(b3s->inst->midicfg, k, v);
+	    dirty = true;
 	  }
 	  b3s->suspend_ui_msg = 0;
 	}
@@ -1158,6 +1176,14 @@ run(LV2_Handle instance, uint32_t n_samples)
     }
     b3s->active_keys[i] = b3s->inst->synth->_activeKeys[i];
   }
+
+  if (dirty) {
+    LV2_Atom_Forge_Frame frame;
+    lv2_atom_forge_frame_time(&b3s->forge, 0);
+    x_forge_object(&b3s->forge, &frame, 1, b3s->uris.state_Changed);
+    lv2_atom_forge_pop(&b3s->forge, &frame);
+  }
+
   if (keychanged) {
     LV2_Atom_Forge_Frame frame;
     lv2_atom_forge_frame_time(&b3s->forge, 0);
@@ -1193,17 +1219,80 @@ cleanup(LV2_Handle instance)
   free(instance);
 }
 
+static char*
+mn_file (LV2_Handle instance)
+{
+  B3S* b3s = (B3S*)instance;
+  char model[16];
+  snprintf (model, 16, "sbf-%p", b3s);
+  model[15] = 0;
+  char* buf = NULL;
+  size_t siz = 0;
+  LOCALEGUARD_START;
+
+#ifdef _WIN32
+  char temppath[MAX_PATH - 13];
+  char filename[MAX_PATH + 1];
+  if (0 == GetTempPath(sizeof(temppath), temppath))
+    return NULL;
+  if (0 == GetTempFileName(temppath, "sbfmidnam", 0, filename))
+    return NULL;
+  FILE *f = fopen(filename, "wb");
+#else
+  FILE* f = open_memstream (&buf, &siz);
+#endif
+  if (!f) {
+    return NULL;
+  }
+  save_midname (b3s->inst, f, model);
+  fclose (f);
+
+#ifdef _WIN32
+  f = fopen(filename, "rb");
+  fseek (f , 0 , SEEK_END);
+  long int rsize = ftell (f);
+  rewind(f);
+  buf = (char*) malloc(rsize);
+  fread(buf, sizeof(char), rsize, f);
+  fclose(f);
+  unlink(filename);
+#endif
+  LOCALEGUARD_END;
+
+  return buf;
+}
+
+static char*
+mn_model (LV2_Handle instance)
+{
+  B3S* b3s = (B3S*)instance;
+  char* rv = (char*) malloc (15 * sizeof (char));
+  snprintf (rv, 16, "sbf-%p", b3s);
+  rv[15] = 0;
+  return rv;
+}
+
+static void
+mn_free (char* v)
+{
+  free (v);
+}
+
 const void*
 extension_data(const char* uri)
 {
   static const LV2_Worker_Interface worker = { work, work_response, NULL };
   static const LV2_State_Interface  state  = { save, restore };
+  static const LV2_Midnam_Interface midnam = { mn_file, mn_model, mn_free };
   if (!strcmp(uri, LV2_WORKER__interface)) {
     return &worker;
   }
   else if (!strcmp(uri, LV2_STATE__interface)) {
     return &state;
   }
+  else if (!strcmp (uri, LV2_MIDNAM__interface)) {
+    return &midnam;
+  }
   return NULL;
 }
 
