@@ -31,6 +31,10 @@
  */
 #ifndef CONFIGDOCONLY
 
+#ifndef _GNU_SOURCE
+#define _GNU_SOURCE
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <fcntl.h>
@@ -41,6 +45,7 @@
 #include <sys/time.h>
 #include <unistd.h>
 #include <pthread.h>
+#include <locale.h>
 #include <assert.h>
 
 #include "main.h"
@@ -560,17 +565,21 @@ int bindToProgram (void * pp,
     break;
 
   case pr_ReverbMix:
-    PGM->flags[0] |= (FL_INUSE|FL_RVBMIX);
-    if (sscanf (val, "%f", &fv) == 0) {
-      sprintf (msg, "Unrecognized reverb mix value : '%s'", val);
-      return stateMessage (fileName, lineNumber, msg, -1);
-    }
-    else if ((fv < 0.0) || (1.0 < fv)) {
-      sprintf (msg, "Reverb mix value out of range : %f", fv);
-      return stateMessage (fileName, lineNumber, msg, -1);
-    }
-    else {
-      PGM->reverbMix = fv;
+    {
+      LOCALEGUARD_START;
+      PGM->flags[0] |= (FL_INUSE|FL_RVBMIX);
+      if (sscanf (val, "%f", &fv) == 0) {
+	sprintf (msg, "Unrecognized reverb mix value : '%s'", val);
+	return stateMessage (fileName, lineNumber, msg, -1);
+      }
+      else if ((fv < 0.0) || (1.0 < fv)) {
+	sprintf (msg, "Reverb mix value out of range : %f", fv);
+	return stateMessage (fileName, lineNumber, msg, -1);
+      }
+      else {
+	PGM->reverbMix = fv;
+      }
+      LOCALEGUARD_END;
     }
     break;
 
@@ -1180,7 +1189,7 @@ void writeProgramm(int pgmNr, Programme *p, const char *sep, FILE * fp) {
   if (p->flags[0] & FL_PRCSPD) { fprintf(fp, "%s, percspeed=%s", sep, p->percussionSpeed ? "fast" : "slow"); }
   if (p->flags[0] & FL_PRCHRM) { fprintf(fp, "%s, percharm=%s", sep, p->percussionHarmonic ? "2nd" : "3rd"); }
   if (p->flags[0] & FL_OVRSEL) { fprintf(fp, "%s, overdrive=%s", sep, p->overdriveSelect ? "off" : "on"); }
-  if (p->flags[0] & FL_RVBMIX) { fprintf(fp, "%s, reverbmix=%f", sep, p->reverbMix); }
+  if (p->flags[0] & FL_RVBMIX) { LOCALEGUARD_START; fprintf(fp, "%s, reverbmix=%f", sep, p->reverbMix); LOCALEGUARD_END;}
   if (p->flags[0] & FL_ROTSPS) {
     fprintf(fp, "%s, rotaryspeed=", sep);
     switch(p->rotarySpeedSelect) {
@@ -1239,6 +1248,7 @@ void hardcode_program (struct b_programme *p, FILE * fp) {
   int i;
   fprintf(fp, "/* generated by programd */\n");
   fprintf(fp, "static const Programme defaultprogrammes[MAXPROGS] = {\n");
+  LOCALEGUARD_START;
   for (i=0 ; i < MAXPROGS; ++i) {
     int j;
     fprintf(fp,"  {");
@@ -1274,6 +1284,7 @@ void hardcode_program (struct b_programme *p, FILE * fp) {
 
     fprintf(fp,"},\n");
   }
+  LOCALEGUARD_END;
   fprintf(fp,"};\n");
 }
 
